#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const commander_1 = require("commander");
const tslog_1 = require("tslog");
const server_1 = require("../lib/server");
const eufy_security_client_1 = require("eufy-security-client");
const program = new commander_1.Command();
program
    .addOption(new commander_1.Option("-c, --config <file>", "Configuration file").default("config.json", "looks in current directory"))
    .addOption(new commander_1.Option("-p, --port <port>", "Listening port").default(3000))
    .addOption(new commander_1.Option("-H, --host <host>", "Listening Host").default("localhost"))
    .addOption(new commander_1.Option("-v, --verbose"))
    .addOption(new commander_1.Option("-q, --quiet"));
program.parse(process.argv);
const args = program.opts();
(async () => {
    let configPath = args.config;
    if (configPath && configPath.substring(0, 1) !== "/") {
        configPath = (0, path_1.resolve)(process.cwd(), configPath);
    }
    let config;
    if (configPath) {
        try {
            config = require(configPath);
            if (config.username === undefined) {
                console.error("Error: Username for Eufy Cloud is missing!");
                return;
            }
            if (config.password === undefined) {
                console.error("Error: Password for Eufy Cloud is missing!");
                return;
            }
        }
        catch (err) {
            console.error(`Error: failed loading config file ${configPath}`);
            console.error(err);
            return;
        }
    }
    else {
        console.error(`Error: failed loading config file ${configPath}`);
        return;
    }
    const logger = new tslog_1.Logger({
        minLevel: args.verbose ? "silly" : "info",
        suppressStdOutput: args.quiet ? true : false,
        displayFilePath: args.verbose ? "hideNodeModulesOnly" : "hidden",
        displayFunctionName: args.verbose ? true : false
    });
    const driver = new eufy_security_client_1.EufySecurity(config, logger);
    let server;
    try {
        server = new server_1.EufySecurityServer(driver, { port: args.port, host: args.host, logger: logger });
        await server.start();
    }
    catch (error) {
        logger.error("Unable to start Server", error);
    }
    let closing = false;
    const handleShutdown = async () => {
        driver.close();
        // Pressing ctrl+c twice.
        if (closing) {
            process.exit();
        }
        // Close gracefully
        closing = true;
        console.log("Shutting down");
        if (server) {
            await server.destroy();
        }
        process.exit();
    };
    process.on("SIGINT", handleShutdown);
    process.on("SIGTERM", handleShutdown);
})().catch((err) => {
    console.error("Unable to start driver", err);
});
//# sourceMappingURL=server.js.map