"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceMessageHandler = void 0;
const error_1 = require("../error");
const command_1 = require("./command");
const event_1 = require("./event");
class DeviceMessageHandler {
    static getStreamingDevices(stationSN) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            return DeviceMessageHandler.streamingDevices[stationSN];
        return [];
    }
    static removeStreamingDevice(stationSN, client) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined)
            DeviceMessageHandler.streamingDevices[stationSN] = DeviceMessageHandler.streamingDevices[stationSN].filter((cl) => cl !== client);
    }
    static addStreamingDevice(stationSN, client) {
        if (DeviceMessageHandler.streamingDevices[stationSN] !== undefined && !DeviceMessageHandler.streamingDevices[stationSN].includes(client))
            DeviceMessageHandler.streamingDevices[stationSN].push(client);
        else if (DeviceMessageHandler.streamingDevices[stationSN] === undefined)
            DeviceMessageHandler.streamingDevices[stationSN] = [client];
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    static async handle(message, driver, client) {
        const { serialNumber, command } = message;
        const device = driver.getDevice(serialNumber);
        const station = driver.getStation(device.getStationSerial());
        switch (command) {
            case command_1.DeviceCommand.setStatusLed:
                await station.setStatusLed(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setAutoNightVision:
                await station.setAutoNightVision(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setMotionDetection:
                await station.setMotionDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setSoundDetection:
                await station.setSoundDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setPetDetection:
                await station.setPetDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setRTSPStream:
                await station.setRTSPStream(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setAntiTheftDetection:
                await station.setAntiTheftDetection(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.setWatermark:
                await station.setWatermark(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.enableDevice:
                await station.enableDevice(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.lockDevice:
                await station.lockDevice(device, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.getPropertiesMetadata:
                {
                    const properties = device.getPropertiesMetadata();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: device.getSerial(),
                            properties: properties
                        };
                    }
                }
            case command_1.DeviceCommand.getProperties:
                {
                    const properties = device.getProperties();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: device.getSerial(),
                            properties: properties
                        };
                    }
                }
            case command_1.DeviceCommand.setProperty:
                await driver.setDeviceProperty(serialNumber, message.name, message.value).catch((error) => {
                    throw error;
                });
                return {};
            case command_1.DeviceCommand.startLivestream:
                if (client.schemaVersion >= 2) {
                    if (!station.isLiveStreaming(device)) {
                        await station.startLivestream(device).catch((error) => {
                            throw error;
                        });
                        client.receiveLivestream[serialNumber] = true;
                        DeviceMessageHandler.addStreamingDevice(station.getSerial(), client);
                    }
                    else if (client.receiveLivestream[serialNumber] !== true) {
                        //TODO: Cache last received I-Frame and send that with the rest to the new client
                        client.sendEvent({
                            source: "device",
                            event: event_1.DeviceEvent.livestreamStarted,
                            serialNumber: serialNumber,
                        });
                        client.receiveLivestream[serialNumber] = true;
                        DeviceMessageHandler.addStreamingDevice(station.getSerial(), client);
                    }
                    else {
                        throw new error_1.LivestreamAlreadyRunningError(`Livestream for device ${serialNumber} is already running`);
                    }
                    return {};
                }
            case command_1.DeviceCommand.stopLivestream:
                if (client.schemaVersion >= 2) {
                    if (!station.isLiveStreaming(device)) {
                        throw new error_1.LivestreamNotRunningError(`Livestream for device ${serialNumber} could not be stopped, because it is not running`);
                    }
                    if (client.receiveLivestream[serialNumber] !== true) {
                        throw new error_1.LivestreamNotRunningError(`This client has not requested the start of the live stream for the device ${serialNumber} and therefore cannot request its termination`);
                    }
                    if (DeviceMessageHandler.streamingDevices[station.getSerial()] !== undefined && DeviceMessageHandler.streamingDevices[station.getSerial()].includes(client)) {
                        if (DeviceMessageHandler.streamingDevices[station.getSerial()].length === 1) {
                            DeviceMessageHandler.removeStreamingDevice(station.getSerial(), client);
                            await station.stopLivestream(device).catch((error) => {
                                throw error;
                            });
                        }
                        else {
                            client.receiveLivestream[serialNumber] = false;
                            DeviceMessageHandler.removeStreamingDevice(station.getSerial(), client);
                            client.sendEvent({
                                source: "device",
                                event: event_1.DeviceEvent.livestreamStopped,
                                serialNumber: serialNumber,
                            });
                        }
                    }
                    return {};
                }
            case command_1.DeviceCommand.isLiveStreaming:
                {
                    if (client.schemaVersion >= 2) {
                        const result = station.isLiveStreaming(device);
                        if (client.schemaVersion >= 2 && client.schemaVersion <= 3) {
                            return { livestreaming: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                livestreaming: result
                            };
                        }
                    }
                }
            case command_1.DeviceCommand.triggerAlarm:
                if (client.schemaVersion >= 3) {
                    await station.triggerDeviceAlarmSound(device, message.seconds).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.resetAlarm:
                if (client.schemaVersion >= 3) {
                    await station.resetDeviceAlarmSound(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.panAndTilt:
                if (client.schemaVersion >= 3) {
                    await station.panAndTilt(device, message.direction).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.quickResponse:
                if (client.schemaVersion >= 3) {
                    await station.quickResponse(device, message.voiceId).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.startDownload:
                if (client.schemaVersion >= 3) {
                    await station.startDownload(device, message.path, message.cipherId).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.cancelDownload:
                if (client.schemaVersion >= 3) {
                    await station.cancelDownload(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.getVoices:
                if (client.schemaVersion >= 3) {
                    const voices = await driver.getApi().getVoices(device.getSerial()).catch((error) => {
                        throw error;
                    });
                    if (client.schemaVersion === 3) {
                        return {
                            serialNumber: device.getSerial(),
                            voices: voices
                        };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: device.getSerial(),
                            voices: voices
                        };
                    }
                }
            case command_1.DeviceCommand.hasProperty:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.hasProperty(message.propertyName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                exists: result
                            };
                        }
                    }
                }
            case command_1.DeviceCommand.hasCommand:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.hasCommand(message.commandName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                exists: result
                            };
                        }
                    }
                }
            case command_1.DeviceCommand.getCommands:
                {
                    if (client.schemaVersion >= 3) {
                        const result = device.getCommands();
                        if (client.schemaVersion === 3) {
                            return { commands: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: device.getSerial(),
                                commands: result
                            };
                        }
                    }
                }
            case command_1.DeviceCommand.startRTSPLivestream:
                if (client.schemaVersion >= 6) {
                    await station.startRTSPStream(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.stopRTSPLivestream:
                if (client.schemaVersion >= 6) {
                    await station.stopRTSPStream(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            case command_1.DeviceCommand.isRTSPLiveStreaming:
                {
                    if (client.schemaVersion >= 6) {
                        const result = station.isRTSPLiveStreaming(device);
                        return {
                            serialNumber: device.getSerial(),
                            livestreaming: result
                        };
                    }
                }
            case command_1.DeviceCommand.calibrateLock:
                if (client.schemaVersion >= 9) {
                    await station.calibrateLock(device).catch((error) => {
                        throw error;
                    });
                    return {};
                }
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.DeviceMessageHandler = DeviceMessageHandler;
DeviceMessageHandler.streamingDevices = {};
//# sourceMappingURL=message_handler.js.map