"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EufySecurityServer = exports.ClientsController = exports.Client = void 0;
const ws_1 = __importDefault(require("ws"));
const tslog_1 = require("tslog");
const events_1 = require("events");
const http_1 = require("http");
const eufy_security_client_1 = require("eufy-security-client");
const forward_1 = require("./forward");
const const_1 = require("./const");
const message_handler_1 = require("./device/message_handler");
const message_handler_2 = require("./station/message_handler");
const error_1 = require("./error");
const instance_1 = require("./instance");
const command_1 = require("./command");
const message_handler_3 = require("./driver/message_handler");
const state_1 = require("./state");
const logging_1 = require("./logging");
class Client {
    constructor(socket, driver, logger, clientsController) {
        this.socket = socket;
        this.driver = driver;
        this.logger = logger;
        this.clientsController = clientsController;
        this.receiveEvents = false;
        this.receiveLogs = false;
        this._outstandingPing = false;
        this.schemaVersion = const_1.minSchemaVersion;
        this.receiveLivestream = {};
        this.instanceHandlers = {
            [instance_1.Instance.station]: (message) => message_handler_2.StationMessageHandler.handle(message, this.driver, this),
            [instance_1.Instance.driver]: (message) => message_handler_3.DriverMessageHandler.handle(message, this.driver, this, this.clientsController, this.logger),
            [instance_1.Instance.device]: (message) => message_handler_1.DeviceMessageHandler.handle(message, this.driver, this),
        };
        socket.on("pong", () => {
            this._outstandingPing = false;
        });
        socket.on("message", (data) => this.receiveMessage(data));
    }
    get isConnected() {
        return this.socket.readyState === this.socket.OPEN;
    }
    async receiveMessage(data) {
        let msg;
        try {
            msg = JSON.parse(data);
        }
        catch (err) {
            // We don't have the message ID. Just close it.
            this.logger.debug(`Unable to parse data: ${data}`);
            this.socket.close();
            return;
        }
        try {
            if (msg.command === command_1.ServerCommand.setApiSchema) {
                // Handle schema version
                this.schemaVersion = msg.schemaVersion;
                if (this.schemaVersion < const_1.minSchemaVersion || this.schemaVersion > const_1.maxSchemaVersion) {
                    throw new error_1.SchemaIncompatibleError(this.schemaVersion);
                }
                this.sendResultSuccess(msg.messageId, {});
                return;
            }
            if (msg.command === command_1.ServerCommand.startListening) {
                this.sendResultSuccess(msg.messageId, {
                    state: (0, state_1.dumpState)(this.driver, this.schemaVersion),
                });
                this.receiveEvents = true;
                return;
            }
            const instance = msg.command.split(".")[0];
            if (this.instanceHandlers[instance]) {
                return this.sendResultSuccess(msg.messageId, await this.instanceHandlers[instance](msg));
            }
            throw new error_1.UnknownCommandError(msg.command);
        }
        catch (error) {
            if (error instanceof error_1.BaseError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error.errorCode);
            }
            if (error instanceof eufy_security_client_1.DeviceNotFoundError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceNotFound);
            }
            if (error instanceof eufy_security_client_1.StationNotFoundError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.stationNotFound);
            }
            if (error instanceof eufy_security_client_1.NotSupportedError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceNotSupported);
            }
            if (error instanceof eufy_security_client_1.WrongStationError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceWrongStation);
            }
            if (error instanceof eufy_security_client_1.InvalidPropertyValueError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceInvalidPropertyValue);
            }
            if (error instanceof eufy_security_client_1.ReadOnlyPropertyError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceReadOnlyProperty);
            }
            if (error instanceof eufy_security_client_1.InvalidCountryCodeError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.invalidCountryCode);
            }
            if (error instanceof eufy_security_client_1.InvalidLanguageCodeError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.invalidLanguageCode);
            }
            if (error instanceof eufy_security_client_1.InvalidPropertyError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceInvalidProperty);
            }
            if (error instanceof error_1.LivestreamAlreadyRunningError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceLivestreamAlreadyRunning);
            }
            if (error instanceof error_1.LivestreamNotRunningError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceLivestreamNotRunning);
            }
            if (error instanceof eufy_security_client_1.PropertyNotSupportedError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.devicePropertyNotSupported);
            }
            if (error instanceof eufy_security_client_1.InvalidCommandValueError) {
                this.logger.error("Message error", error);
                return this.sendResultError(msg.messageId, error_1.ErrorCode.deviceInvalidCommandValue);
            }
            this.logger.error("Unexpected error", error);
            this.sendResultError(msg.messageId, error_1.ErrorCode.unknownError);
        }
    }
    sendVersion() {
        this.sendData({
            type: "version",
            driverVersion: eufy_security_client_1.libVersion,
            serverVersion: const_1.version,
            minSchemaVersion: const_1.minSchemaVersion,
            maxSchemaVersion: const_1.maxSchemaVersion,
        });
    }
    sendResultSuccess(messageId, result) {
        this.sendData({
            type: "result",
            success: true,
            messageId,
            result,
        });
    }
    sendResultError(messageId, errorCode) {
        this.sendData({
            type: "result",
            success: false,
            messageId,
            errorCode,
        });
    }
    sendEvent(event) {
        this.sendData({
            type: "event",
            event,
        });
    }
    sendData(data) {
        this.socket.send(JSON.stringify(data));
    }
    checkAlive() {
        if (this._outstandingPing) {
            this.disconnect();
            return;
        }
        this._outstandingPing = true;
        this.socket.ping();
    }
    disconnect() {
        this.socket.close();
    }
}
exports.Client = Client;
class ClientsController {
    constructor(driver, logger) {
        this.driver = driver;
        this.logger = logger;
        this.clients = [];
        this.cleanupScheduled = false;
    }
    addSocket(socket, request) {
        this.logger.debug(`New client with ip: ${request.socket.remoteAddress} port: ${request.socket.remotePort}`);
        const client = new Client(socket, this.driver, this.logger, this);
        socket.on("error", (error) => {
            this.logger.error(`Client with ip: ${request.socket.remoteAddress} port: ${request.socket.remotePort} socket error`, error);
        });
        socket.on("close", (code, reason) => {
            this.logger.info(`Client disconnected with ip: ${request.socket.remoteAddress} port: ${request.socket.remotePort} code: ${code} reason: ${reason}`);
            this.scheduleClientCleanup();
        });
        client.sendVersion();
        this.clients.push(client);
        if (this.pingInterval === undefined) {
            this.pingInterval = setInterval(() => {
                const newClients = [];
                for (const client of this.clients) {
                    if (client.isConnected) {
                        newClients.push(client);
                    }
                    else {
                        client.disconnect();
                    }
                }
                this.clients = newClients;
            }, 30000);
        }
        if (this.eventForwarder === undefined) {
            this.eventForwarder = new forward_1.EventForwarder(this);
            this.eventForwarder.start();
        }
    }
    get loggingEventForwarderStarted() {
        var _a;
        return ((_a = this.loggingEventForwarder) === null || _a === void 0 ? void 0 : _a.started) === true;
    }
    restartLoggingEventForwarderIfNeeded() {
        var _a;
        (_a = this.loggingEventForwarder) === null || _a === void 0 ? void 0 : _a.restartIfNeeded();
    }
    configureLoggingEventForwarder() {
        var _a;
        if (this.loggingEventForwarder === undefined) {
            this.loggingEventForwarder = new logging_1.LoggingEventForwarder(this, this.logger);
        }
        if (!this.loggingEventForwarderStarted) {
            (_a = this.loggingEventForwarder) === null || _a === void 0 ? void 0 : _a.start();
        }
    }
    cleanupLoggingEventForwarder() {
        var _a;
        if (this.clients.filter((cl) => cl.receiveLogs).length == 0 && this.loggingEventForwarderStarted) {
            (_a = this.loggingEventForwarder) === null || _a === void 0 ? void 0 : _a.stop();
        }
    }
    scheduleClientCleanup() {
        if (this.cleanupScheduled) {
            return;
        }
        this.cleanupScheduled = true;
        setTimeout(() => this.cleanupClients(), 0);
    }
    cleanupClients() {
        this.cleanupScheduled = false;
        const disconnectedClients = this.clients.filter((cl) => cl.isConnected === false);
        this.clients = this.clients.filter((cl) => cl.isConnected);
        disconnectedClients.forEach(client => {
            Object.keys(client.receiveLivestream).forEach(serialNumber => {
                try {
                    const device = this.driver.getDevice(serialNumber);
                    const station = this.driver.getStation(device.getStationSerial());
                    const streamingDevices = message_handler_1.DeviceMessageHandler.getStreamingDevices(station.getSerial());
                    if (client.receiveLivestream[serialNumber] === true && streamingDevices.length === 1 && streamingDevices.includes(client)) {
                        if (station.isLiveStreaming(device))
                            station.stopLivestream(device);
                    }
                    client.receiveLivestream[device.getSerial()] = false;
                    message_handler_1.DeviceMessageHandler.removeStreamingDevice(station.getSerial(), client);
                }
                catch (error) {
                    this.logger.error(`Error doing cleanup of client`, error);
                }
            });
        });
        this.cleanupLoggingEventForwarder();
    }
    disconnect() {
        if (this.pingInterval !== undefined) {
            clearInterval(this.pingInterval);
        }
        this.pingInterval = undefined;
        this.clients.forEach((client) => client.disconnect());
        this.clients = [];
        this.cleanupLoggingEventForwarder();
    }
}
exports.ClientsController = ClientsController;
class EufySecurityServer extends events_1.EventEmitter {
    constructor(driver, options) {
        var _a;
        super();
        this.driver = driver;
        this.options = options;
        this.logger = (_a = options.logger) !== null && _a !== void 0 ? _a : new tslog_1.Logger();
    }
    async start() {
        this.server = (0, http_1.createServer)();
        this.wsServer = new ws_1.default.Server({ server: this.server });
        this.sockets = new ClientsController(this.driver, this.logger);
        this.wsServer.on("connection", (socket, request) => { var _a; return (_a = this.sockets) === null || _a === void 0 ? void 0 : _a.addSocket(socket, request); });
        this.logger.debug(`Starting server on host ${this.options.host}, port ${this.options.port}`);
        this.server.on("error", this.onError.bind(this));
        this.server.listen(this.options.port, this.options.host);
        await (0, events_1.once)(this.server, "listening");
        this.emit("listening");
        this.logger.info(`Eufy Security server listening on host ${this.options.host}, port ${this.options.port}`);
        this.driver.connect();
    }
    onError(error) {
        this.emit("error", error);
        this.logger.error(error);
    }
    async destroy() {
        this.logger.debug(`Closing server...`);
        if (this.sockets) {
            this.sockets.disconnect();
        }
        if (this.server) {
            this.server.close();
            await (0, events_1.once)(this.server, "close");
        }
        this.logger.info(`Server closed`);
    }
}
exports.EufySecurityServer = EufySecurityServer;
//# sourceMappingURL=server.js.map